// This is a C++ source file.

// Use the BOSS macros to describe the layout of a shifter cell.
// You should be reasonably familiar with OCT and the OCT Symbolic
// policy to understand fully this description.

// The description is constructive and procedural, one piece at a time.
// It helps to draw a colored stick diagram to understand the description.
// 

#include "boss.h"		



void bossAppShiftCell()
{
    // Declare the output facet.
    BossFacet facet = BossFacet("shift_cell", "symbolic"); 	

    // Start with an n-type 4 by 2 transitor.
    BossTransistor n_inv( facet, "nmos", 4, 2);
    
    // Add some stubs to source and drain of the transistor.
    // ( A stub is a short segment with a contact )
    BossContact& n_inv_cd = bossStub( n_inv.term("SOURCE"), BOSS_DOWN, 140 );
    BossContact& n_inv_cu = bossStub( n_inv.term("DRAIN"), BOSS_UP );

    // Add the MET1 layer to the lower contact.
    n_inv_cd.add_layer("MET1");  
    
    
    //
    // Another  4x2 p transistor.
    // Place the p transistor above n transistor
    // and the two transistors.
    // Then add the stubs to source and drain.
    //
    BossTransistor p_inv(facet, "pmos", 4, 2);
    p_inv.above( n_inv , 300 );					
    p_inv.align_x( n_inv );
    BossContact& p_inv_cu = bossStub( p_inv.term("DRAIN"), BOSS_UP, 140 );
    BossContact& p_inv_cd = bossStub( p_inv.term("SOURCE"), BOSS_DOWN, -20 );
    p_inv_cu.add_layer("MET1");


    //
    // Connect two connectors with a straight segment in MET1.
    // 
    bossStraightRoute( n_inv_cu, p_inv_cd, 60, "MET1");
    
    //
    // Create, rotate, align two more transistors.
    // 
    BossTransistor p_clk(facet, "pmos", 4, 2);
    p_clk.transform(OCT_ROT90);
    p_clk.right_of( p_inv_cd, 40);
    p_clk.align_y( p_inv_cd);
    BossTransistor n_clk(facet, "nmos", 4, 2);
    n_clk.transform(OCT_ROT90);
    n_clk.align_x(p_clk);
    n_clk.align_y(n_inv_cu);
    

    // 
    // Connect the drains of the two new transistors.
    // 
    bossStraightRoute( n_inv_cu, n_clk.term( "DRAIN" ), 80, "NDIF");
    bossStraightRoute( p_inv_cd, p_clk.term( "DRAIN" ), 80, "PDIF");
    

    //
    // Create a generic contact for routing. Place it appropriately
    // and connect it to the two clk transistors. Notice that the connections are
    // not straight, but have a Z shape.
    // 
    // 
    BossContact out_m1_po(facet);
    out_m1_po.right_of(n_clk, 100);
    out_m1_po.between_y(n_clk, p_clk);
    bossZRoute( n_clk.term("SOURCE"), out_m1_po, 80, BOSS_VERTICAL, "NDIF", "MET1", "MET1");
    bossZRoute( p_clk.term("SOURCE"), out_m1_po, 80, BOSS_VERTICAL, "PDIF", "MET1", "MET1");
    out_m1_po.add_layer("POLY");

    
    //
    //
    //
    BossContact& n_inv_cl = bossStub( n_inv.term("GATE"), BOSS_LEFT, 40);  
    BossContact& p_inv_cl = bossStub( p_inv.term("GATE"), BOSS_LEFT, 40);  
    BossContact& input = bossStub( n_inv_cl, BOSS_UP, 100);
    bossStraightRoute( input, p_inv_cl, 40, "POLY");

    
    //
    //
    //
    BossContact& top_clk_po1 = bossStub( p_clk.term("GATE1"), BOSS_UP, -40 );  
    BossContact& top_clk_po2 = bossStub( top_clk_po1, BOSS_RIGHT, 0 );  
    BossContact& top_clk_p_m1 = bossStub( top_clk_po2, BOSS_UP, 20 );  
    BossContact& top_clk_m1_m2 = bossStub( top_clk_p_m1, BOSS_LEFT, 40, "MET1");
    top_clk_m1_m2.add_layer("MET2");
    
    BossContact& bot_clk_po = bossStub( n_clk.term("GATE"), BOSS_DOWN, 80 );  
    BossContact& bot_clk_m1 = bossStub( bot_clk_po, BOSS_RIGHT, 60, "MET1");
    bot_clk_m1.add_layer("MET2");



    //
    // Define the formal terminals of this facet.
    //
    BossFormalTerm gnd_f( n_inv_cd, "GND");
    BossFormalTerm vdd_f( p_inv_cu, "Vdd");
    BossFormalTerm out_f( out_m1_po, "out");
    BossFormalTerm in_f( input, "in");
    BossFormalTerm phi_f( top_clk_m1_m2, "phi");
    BossFormalTerm phi_bar_f( bot_clk_m1, "phi_bar");
    
    //
    // Add constraints for compaction with sparcs.
    // Look at $OCTTOOLS/doc/symlib.doc for more info on these functions. 
    //
    bossClearConstraints(facet);
    bossSetConstraint(BOSS_HORIZONTAL, top_clk_m1_m2, bot_clk_m1, 160, SYM_PUT_CONST_UNDEFINED);
    bossSetConstraint(BOSS_VERTICAL, n_inv_cd, bot_clk_m1, 140, SYM_PUT_CONST_UNDEFINED);
    bossSetConstraint(BOSS_VERTICAL, top_clk_m1_m2, p_inv_cu, 140, SYM_PUT_CONST_UNDEFINED);
}

main( int argc, char** argv )
{
    // Required: first call in main().
    bossBegin( argc, argv );

    bossAppShiftCell();

    // Do not call any other BOSS function in main():
    // call only bossBegin() and bossEnd().
    // 
    // bossEnd() does not return. It is like calling exit().
    // This is required: it should be the last call in main().
    bossEnd( 0 );
}

